@@ -100,32 +100,6 @@ dungeon ()
     {
       /* Increment turn counter                 */
       turn++;
-      /* The Mac ignores the game hours file            */
-      /* Check for game hours                          */
-      if (((turn % 250) == 1) && !check_time ())
-	{
-	  if (closing_flag > 4)
-	    {
-	      msg_print ("The gates to Moria are now closed.");
-	      snprintf (died_from, sizeof (died_from), "%s", 
-			"(closing gate: saved)");
-	      if (!save_char ())
-		{
-		  snprintf (died_from, sizeof (died_from), "%s",
-			    "a slammed gate");
-
-		  death = TRUE;
-		}
-	      exit_game ();
-	    }
-	  else
-	    {
-	      disturb (0, 0);
-	      closing_flag++;
-	      msg_print ("The gates to Moria are closing.");
-	      msg_print ("Please finish up or save your game.");
-	    }
-	}
 
       /* turn over the store contents every, say, 1000 turns */
       if ((dun_level != 0) && ((turn % 1000) == 0))
@@ -1843,7 +1817,7 @@ examine_book ()
 {
   int32u j;
   int i, k, item_val, flag;
-  int spell_index[31];
+  int spell_index[MAX_SPELLS];
   register inven_type *i_ptr;
   register spell_type *s_ptr;
 
@@ -1877,17 +1851,14 @@ examine_book ()
       else
 	{
 	  i = 0;
-	  j = inventory[item_val].flags;
-	  while (j)
-	    {
-	      k = bit_pos (&j);
-	      s_ptr = &magic_spell[py.misc.pclass][k];
-	      if (s_ptr->slevel < 99)
-		{
-		  spell_index[i] = k;
-		  i++;
-		}
-	    }
+          for (j = 0; j < MAX_SPELLS; j++)
+            {
+              if (object_list[inventory[item_val].index].spells[j])
+                {
+                  spell_index[i] = j;
+                  i++;
+                }
+            }
 	  save_screen ();
 	  print_spells (spell_index, i, TRUE, -1);
 	  pause_line (0);
