@@ -75,22 +75,6 @@ reset_seed ()
 }
 
 
-/* Check the day-time strings to see if open		-RAK-	*/
-int
-check_time ()
-{
-  long clock_var;
-  register struct tm *tp;
-
-  clock_var = time ((long *) 0);
-  tp = localtime (&clock_var);
-  if (days[tp->tm_wday][tp->tm_hour + 4] == 'X')
-    return TRUE;
-  else
-    return FALSE;
-}
-
-
 /* Generates a random integer x where 1<=X<=MAXVAL	-RAK-	*/
 int
 randint (maxval)
