@@ -1587,7 +1587,7 @@ print_spells (spell, num, comment, nonconsec)
   else
     col = 31;
   offset =
-    (class[py.misc.pclass].spell == MAGE ? SPELL_OFFSET : PRAYER_OFFSET);
+    (class[py.misc.pclass].spell == MAGE ? 0 : 1);
   erase_line (1, col);
   put_buffer ("Name", 1, col + 5);
   put_buffer ("Lv Mana Fail", 1, col + 35);
@@ -1600,11 +1600,11 @@ print_spells (spell, num, comment, nonconsec)
       s_ptr = &magic_spell[py.misc.pclass][j];
       if (comment == FALSE)
 	p = "";
-      else if ((spell_forgotten & (1L << j)) != 0)
+      else if (spell_status[j].forgotten == 1)
 	p = " forgotten";
-      else if ((spell_learned & (1L << j)) == 0)
+      else if (spell_status[j].learned == 0)
 	p = " unknown";
-      else if ((spell_worked & (1L << j)) == 0)
+      else if (spell_status[j].worked == 0)
 	p = " untried";
       else
 	p = "";
@@ -1617,7 +1617,7 @@ print_spells (spell, num, comment, nonconsec)
 	spell_char = 'a' + j - nonconsec;
       snprintf (out_val, sizeof (out_val),
 		"  %c) %-30s%2d %4d %3d%%%s", spell_char,
-		spell_names[j + offset], s_ptr->slevel, s_ptr->smana,
+		spell_names[offset][j], s_ptr->slevel, s_ptr->smana,
 		spell_chance (j), p);
       prt (out_val, 2 + i, col);
     }
@@ -1646,7 +1646,7 @@ get_spell (spell, num, sn, sc, prompt, first_spell)
 	    spell[num - 1] + 'a' - first_spell, prompt);
   redraw = FALSE;
   offset =
-    (class[py.misc.pclass].spell == MAGE ? SPELL_OFFSET : PRAYER_OFFSET);
+    (class[py.misc.pclass].spell == MAGE ? 0: 1);
   while (flag == FALSE && get_com (out_str, &choice))
     {
       if (isupper ((int) choice))
@@ -1663,7 +1663,7 @@ get_spell (spell, num, sn, sc, prompt, first_spell)
 	      s_ptr = &magic_spell[py.misc.pclass][*sn];
 	      snprintf (tmp_str, sizeof (tmp_str),
 			"Cast %s (%d mana, %d%% fail)?",
-			spell_names[*sn + offset], s_ptr->smana,
+			spell_names[offset][*sn], s_ptr->smana,
 			spell_chance (*sn));
 	      if (get_check (tmp_str))
 		flag = TRUE;
@@ -1704,7 +1704,7 @@ get_spell (spell, num, sn, sc, prompt, first_spell)
 	{
 	  snprintf (tmp_str, sizeof (tmp_str),
 		    "You don't know that %s.",
-		    (offset == SPELL_OFFSET ? "spell" : "prayer"));
+		    (offset == 0 ? "spell" : "prayer"));
 	  msg_print (tmp_str);
 	}
     }
@@ -1728,7 +1728,7 @@ calc_spells (stat)
   register int i;
   register int32u mask;
   int32u spell_flag;
-  int j, offset;
+  int j, k, offset;
   int num_allowed = 0, new_spells, num_known, levels;
   vtype tmp_str;
   char *p;
@@ -1740,30 +1740,29 @@ calc_spells (stat)
   if (stat == A_INT)
     {
       p = "spell";
-      offset = SPELL_OFFSET;
+      offset = 0;
     }
   else
     {
       p = "prayer";
-      offset = PRAYER_OFFSET;
+      offset = 1;
     }
 
   /* check to see if know any spells greater than level, eliminate them */
-  for (i = 31, mask = 0x80000000L; mask; mask >>= 1, i--)
-    if (mask & spell_learned)
-      {
-	if (msp_ptr[i].slevel > p_ptr->lev)
-	  {
-	    spell_learned &= ~mask;
-	    spell_forgotten |= mask;
-	    snprintf (tmp_str, sizeof (tmp_str),
-		      "You have forgotten the %s of %s.", p,
-		      spell_names[i + offset]);
-	    msg_print (tmp_str);
+  for (i = MAX_SPELLS-1; i >= 0; i--)
+    {
+      if (spell_status[i].learned && msp_ptr[i].slevel > p_ptr->lev)
+        {
+          spell_status[i].learned = 0;
+          spell_status[i].forgotten = 1;
+	  snprintf (tmp_str, sizeof (tmp_str),
+                    "You have forgotten the %s of %s.", p,
+                    spell_names[offset][i]);
+          msg_print (tmp_str);
 	  }
 	else
 	  break;
-      }
+    }
 
   /* calc number of spells allowed */
   levels = p_ptr->lev - class[p_ptr->pclass].first_spell_lev + 1;
@@ -1790,41 +1789,41 @@ calc_spells (stat)
     }
 
   num_known = 0;
-  for (mask = 0x1; mask; mask <<= 1)
-    if (mask & spell_learned)
-      num_known++;
+  for (i = 0; i < MAX_SPELLS; i++)
+    {
+      if (spell_status[i].learned)
+        num_known++;
+    }
+
   new_spells = num_allowed - num_known;
 
   if (new_spells > 0)
     {
       /* remember forgotten spells while forgotten spells exist of new_spells
          positive, remember the spells in the order that they were learned */
-      for (i = 0; (spell_forgotten && new_spells
-		   && (i < num_allowed) && (i < 32)); i++)
+      for (i = 0; i < MAX_SPELLS; i++)
 	{
-	  /* j is (i+1)th spell learned */
+	  // j is (i+1)th spell learned 
 	  j = spell_order[i];
-	  /* shifting by amounts greater than number of bits in long gives
-	     an undefined result, so don't shift for unknown spells */
-	  if (j == 99)
-	    mask = 0x0;
-	  else
-	    mask = 1L << j;
-	  if (mask & spell_forgotten)
+	  if (spell_status[j].forgotten)
 	    {
 	      if (msp_ptr[j].slevel <= p_ptr->lev)
 		{
 		  new_spells--;
-		  spell_forgotten &= ~mask;
-		  spell_learned |= mask;
+                  spell_status[j].forgotten = 0;
+                  spell_status[j].learned = 1;
 		  snprintf (tmp_str, sizeof (tmp_str),
 			    "You have remembered the %s of %s.", p,
-			    spell_names[j + offset]);
+			    spell_names[offset][j]);
 		  msg_print (tmp_str);
 		}
 	      else
 		num_allowed++;
 	    }
+          if (new_spells == 0)
+            break;
+          if (i >= num_allowed)
+            break;
 	}
 
       if (new_spells > 0)
@@ -1832,44 +1831,35 @@ calc_spells (stat)
 	  /* determine which spells player can learn */
 	  /* must check all spells here, in gain_spell() we actually check
 	     if the books are present */
-	  spell_flag = 0x7FFFFFFFL & ~spell_learned;
-
-	  mask = 0x1;
-	  i = 0;
-	  for (j = 0, mask = 0x1; spell_flag; mask <<= 1, j++)
-	    if (spell_flag & mask)
-	      {
-		spell_flag &= ~mask;
-		if (msp_ptr[j].slevel <= p_ptr->lev)
-		  i++;
-	      }
-
-	  if (new_spells > i)
-	    new_spells = i;
+          k = 0;
+          for (i = 0; i < MAX_SPELLS; i++)
+            {
+              if (spell_status[i].learned == 0 &&
+                  msp_ptr[i].slevel <= p_ptr->lev)
+                k++;
+            }
+	  if (new_spells > k)
+	    new_spells = k;
 	}
     }
   else if (new_spells < 0)
     {
-      /* forget spells until new_spells zero or no more spells know, spells
+      /*forget spells until new_spells zero or no more spells known, spells
          are forgotten in the opposite order that they were learned */
-      for (i = 31; new_spells && spell_learned; i--)
+      for (i = MAX_SPELLS; new_spells; i--)
 	{
-	  /* j is the (i+1)th spell learned */
+	  //j is the (i+1)th spell learned 
 	  j = spell_order[i];
-	  /* shifting by amounts greater than number of bits in long gives
-	     an undefined result, so don't shift for unknown spells */
-	  if (j == 99)
-	    mask = 0x0;
-	  else
-	    mask = 1L << j;
-	  if (mask & spell_learned)
+	  // shifting by amounts greater than number of bits in long gives
+	  // an undefined result, so don't shift for unknown spells 
+	  if (spell_status[j].learned)
 	    {
-	      spell_learned &= ~mask;
-	      spell_forgotten |= mask;
+	      spell_status[j].learned = 0;
+	      spell_status[j].forgotten = 1;
 	      new_spells++;
 	      snprintf (tmp_str, sizeof (tmp_str),
 			"You have forgotten the %s of %s.", p,
-			spell_names[j + offset]);
+			spell_names[offset][j]);
 	      msg_print (tmp_str);
 	    }
 	}
@@ -1891,6 +1881,16 @@ calc_spells (stat)
     }
 }
 
+/* check if we have a given spell (by index) in a book of the given tval -BA */
+int has_spell_in_book(int spell_idx, int tval)
+{
+  int i;
+  int has_spell = 0;
+  for (i = 0; i < inven_ctr; i++)
+    if (inventory[i].tval == tval)
+      has_spell |= object_list[inventory[i].index].spells[spell_idx];
+  return has_spell;
+}
 
 /* gain spells when player wants to		- jw */
 void
@@ -1898,7 +1898,7 @@ gain_spells ()
 {
   char query;
   int stat, diff_spells, new_spells;
-  int spells[31], offset, last_known;
+  int spells[MAX_SPELLS], offset, last_known;
   register int i, j;
   register int32u spell_flag, mask;
   vtype tmp_str;
@@ -1921,7 +1921,7 @@ gain_spells ()
   if (class[p_ptr->pclass].spell == MAGE)
     {
       stat = A_INT;
-      offset = SPELL_OFFSET;
+      offset = 0;
 
       /* People with MAGE spells can't learn spells if they can't read their
          books.  */
@@ -1936,14 +1936,21 @@ gain_spells ()
 	  return;
 	}
     }
-  else
+  else if (class[p_ptr->pclass].spell == PRIEST)
     {
       stat = A_WIS;
-      offset = PRAYER_OFFSET;
+      offset = 1;
+    }
+  else
+    {
+      snprintf (tmp_str, sizeof (tmp_str), "You can't learn spells!");
+      msg_print (tmp_str);
+      free_turn_flag = TRUE;
+      return;
     }
 
-  for (last_known = 0; last_known < 32; last_known++)
-    if (spell_order[last_known] == 99)
+  for (last_known = 0; last_known < MAX_SPELLS; last_known++)
+    if (spell_order[last_known] == MAX_SPELLS)
       break;
 
   if (!new_spells)
@@ -1958,31 +1965,20 @@ gain_spells ()
     {
       /* determine which spells player can learn */
       /* mages need the book to learn a spell, priests do not need the book */
-      if (stat == A_INT)
-	{
-	  spell_flag = 0;
-	  for (i = 0; i < inven_ctr; i++)
-	    if (inventory[i].tval == TV_MAGIC_BOOK)
-	      spell_flag |= inventory[i].flags;
-	}
-      else
-	spell_flag = 0x7FFFFFFF;
-
-      /* clear bits for spells already learned */
-      spell_flag &= ~spell_learned;
-
-      mask = 0x1;
       i = 0;
-      for (j = 0, mask = 0x1; spell_flag; mask <<= 1, j++)
-	if (spell_flag & mask)
-	  {
-	    spell_flag &= ~mask;
-	    if (msp_ptr[j].slevel <= p_ptr->lev)
-	      {
-		spells[i] = j;
-		i++;
-	      }
-	  }
+      for (j = 0; j < MAX_SPELLS; j++)
+        {
+          if (spell_status[j].learned == 0 &&
+              msp_ptr[j].slevel <= p_ptr->lev)
+            {
+              if (stat == A_INT && has_spell_in_book(j, TV_MAGIC_BOOK))
+                {
+                  spells[i] = j;
+                  i++;
+                }
+            }
+
+        }
 
       if (new_spells > i)
 	{
@@ -2005,7 +2001,7 @@ gain_spells ()
 	      if (j >= 0 && j < i && j < 22)
 		{
 		  new_spells--;
-		  spell_learned |= 1L << spells[j];
+                  spell_status[j].learned = 1;
 		  spell_order[last_known++] = spells[j];
 		  for (; j <= i - 1; j++)
 		    spells[j] = spells[j + 1];
@@ -2024,11 +2020,11 @@ gain_spells ()
 	  while (new_spells)
 	    {
 	      j = randint (i) - 1;
-	      spell_learned |= 1L << spells[j];
+              spell_status[j].learned = 1;
 	      spell_order[last_known++] = spells[j];
 	      snprintf (tmp_str, sizeof (tmp_str),
 			"You have learned the prayer of %s.",
-			spell_names[spells[j] + offset]);
+			spell_names[offset][spells[j]]);
 	      msg_print (tmp_str);
 	      for (; j <= i - 1; j++)
 		spells[j] = spells[j + 1];
@@ -2057,7 +2053,12 @@ calc_mana (stat)
   register int32 value;
 
   p_ptr = &py.misc;
-  if (spell_learned != 0)
+  int i, num_spells_learned = 0;
+  for (i = 0; i < MAX_SPELLS; i++)
+    if (spell_status[i].learned)
+      num_spells_learned++;
+
+  if (num_spells_learned > 0)
     {
       levels = p_ptr->lev - class[p_ptr->pclass].first_spell_lev + 1;
       switch (stat_adj (stat))
