@@ -245,16 +245,30 @@ cast_spell (prompt, item_val, sn, sc)
 {
   int32u j;
   register int i, k;
-  int spell[31], result, first_spell;
+  int spell[MAX_SPELLS], result, first_spell;
   register spell_type *s_ptr;
 
   result = -1;
   i = 0;
   j = inventory[item_val].flags;
-  first_spell = bit_pos (&j);
-  /* set j again, since bit_pos modified it */
-  j = inventory[item_val].flags & spell_learned;
+  for (j = 0; j < MAX_SPELLS; j++)
+    {
+      if (object_list[inventory[item_val].index].spells[j])
+        {
+          first_spell = j;
+          break;
+        }
+    }
   s_ptr = magic_spell[py.misc.pclass];
+  for (j = 0; j < MAX_SPELLS; j++)
+    {
+      if (s_ptr[j].slevel <= py.misc.lev && spell_status[j].learned)
+        {
+	  spell[i] = j;
+	  i++;
+        }
+    }
+  /*
   while (j)
     {
       k = bit_pos (&j);
@@ -264,6 +278,7 @@ cast_spell (prompt, item_val, sn, sc)
 	  i++;
 	}
     }
+    */
   if (i > 0)
     {
       result = get_spell (spell, i, sn, sc, prompt, first_spell);
