@@ -24,6 +24,7 @@
 #include "types.h"
 
 #include <string.h>
+#include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
 
@@ -39,6 +40,7 @@ static void wr_long ();
 static void wr_bytes ();
 static void wr_string ();
 static void wr_shorts ();
+static void wr_longs ();
 static void wr_item ();
 static void wr_monster ();
 static void rd_byte ();
@@ -47,6 +49,7 @@ static void rd_long ();
 static void rd_bytes ();
 static void rd_string ();
 static void rd_shorts ();
+static void rd_longs ();
 static void rd_item ();
 static void rd_monster ();
 
@@ -238,10 +241,9 @@ sv_write ()
     wr_item (&inventory[i]);
   wr_short ((int16u) inven_weight);
   wr_short ((int16u) equip_ctr);
-  wr_long (spell_learned);
-  wr_long (spell_worked);
-  wr_long (spell_forgotten);
-  wr_bytes (spell_order, 32);
+  wr_long (MAX_SPELLS);
+  wr_longs (spell_order, MAX_SPELLS);
+  wr_longs ((int32u*)spell_status, MAX_SPELLS);
   wr_bytes (object_ident, OBJECT_IDENT_SIZE);
   wr_long (randes_seed);
   wr_long (town_seed);
@@ -494,6 +496,7 @@ get_char (generate)
   store_type *st_ptr;
   int8u char_tmp, ychar, xchar, count;
   int8u version_maj, version_min, patch_level;
+  int32u max_spells;
 
   nosignals ();
   *generate = TRUE;
@@ -759,10 +762,11 @@ get_char (generate)
 	    rd_item (&inventory[i]);
 	  rd_short ((int16u *) & inven_weight);
 	  rd_short ((int16u *) & equip_ctr);
-	  rd_long (&spell_learned);
-	  rd_long (&spell_worked);
-	  rd_long (&spell_forgotten);
-	  rd_bytes (spell_order, 32);
+          rd_long (&max_spells);
+          if (max_spells > MAX_SPELLS)
+            max_spells = MAX_SPELLS;
+          rd_longs (spell_order, max_spells);
+          rd_longs ((int32u*)spell_status, max_spells);
 	  rd_bytes (object_ident, OBJECT_IDENT_SIZE);
 	  rd_long (&randes_seed);
 	  rd_long (&town_seed);
@@ -1128,6 +1132,28 @@ wr_shorts (s, count)
     }
 }
 
+static void
+wr_longs (s, count)
+     int32u *s;
+     register int count;
+{
+  register int i;
+  register int32u *sptr;
+
+  sptr = s;
+  for (i = 0; i < count; i++)
+    {
+      xor_byte ^= (*sptr & 0xFF);
+      putc ((int) xor_byte, fileptr);
+      xor_byte ^= ((*sptr >> 8) & 0xFF);
+      putc ((int) xor_byte, fileptr);
+      xor_byte ^= ((*sptr >> 16) & 0xFF);
+      putc ((int) xor_byte, fileptr);
+      xor_byte ^= ((*sptr++ >> 24) & 0xFF);
+      putc ((int) xor_byte, fileptr);
+    }
+}
+
 static void
 wr_item (item)
      register inven_type *item;
@@ -1265,10 +1291,36 @@ rd_shorts (ptr, count)
     }
 }
 
+static void
+rd_longs (ptr, count)
+     int32u *ptr;
+     register int count;
+{
+  register int i;
+  register int32u *sptr;
+  register int32u s;
+  int8u c;
+
+  sptr = ptr;
+  for (i = 0; i < count; i++)
+    {
+      c = (getc (fileptr) & 0xFF);
+      s = c ^ xor_byte;
+      xor_byte = (getc (fileptr) & 0xFF);
+      s |= (int32u) (c ^ xor_byte) << 8;
+      xor_byte = (getc (fileptr) & 0xFF);
+      s |= (int32u) (c ^ xor_byte) << 16;
+      xor_byte = (getc (fileptr) & 0xFF);
+      s |= (int32u) (c ^ xor_byte) << 24;
+      *sptr++ = s;
+    }
+}
+
 static void
 rd_item (item)
      register inven_type *item;
 {
+  int max_spells;
   rd_short (&item->index);
   rd_byte (&item->name2);
   rd_string (item->inscrip);
